-- --------------------------------------------------------------------------
-- Read a Switch on GPIO-6 and Toggle a Light on GPIO-7
-- --------------------------------------------------------------------------




-- --------------------------------------------------------------------------
-- This is an example program written in Euphoria.
-- Euphoria is a High-Level interpreted language that is easy to understand,
-- and provides a quick means of writing programs for testing and debugging 
-- hardware specific issues.
--
-- Note:  Two dashes (--) are used to start a comment
--        HEX numbers are prefixed with the pound sign (#)
-- --------------------------------------------------------------------------



-- --------------------------------------------------------------------------
-- This program demonstrates how to use the GPIO features of the ZFx86 chip.
--
-- It will configure GPIO-6 as an input, and monitor the status of a switch
-- connected to it.  When the switch is pressed (and then released), the 
-- rising edge of the signal will be used to trigger an event.
--
-- When the switch activity is detected, it toggles the state of a simulated
-- alarm system.  The first press Activates the alarm, sending positive 
-- pulses out on GPIO-7.  These pulses could flash a light, or sound a 
-- buzzer etc.  When the switch is pressed again, the alarm is disabled.
-- When disabled, the GPIO-7 pin is set to a constant zero or low state.
--
-- There are actually two methods of reading GPIO inputs.  One of them reads
-- the current instantaneous situation, and answers the question "Is the
-- input active right now?"  The second technique uses a built-in latch to
-- capture activity of the input.  It answers the question "Has an input 
-- event taken place since the last time I checked?"  Using the configuration
-- developed later in this program, the following statement checks the input
-- at the current moment : if and_bits(inp32(DataIn0), #40) = 0 then ...
-- If it does equal 0, the switch is currently being pressed.
-- The statement : if and_bits(inp32(Status0), #40) != 0 then ...
-- would indicate the event has occurred, but the precise time is unknown.
-- The best technique to use will be determined by the needs of the product.
-- --------------------------------------------------------------------------



-- --------------------------------------------------------------------------
-- Many of the GPIO pins share functionality with other features of the chip.
-- GPIO-6 which is used in this example, is shared with the Secondary IDE
-- Interface, as are several other GPIO pins.  The pins for GPIO-0 and GPIO-4
-- are shared (at times) with the 32KHz and 14MHz clock signals.  When using
-- the GPIO features, precautions must be taken to insure they will not 
-- interfere with, or be disrupted by other on-board systems. 
-- --------------------------------------------------------------------------




-- --------------------------------------------------------------------------
-- Library Routines to Include
-- --------------------------------------------------------------------------
include ports32.e                       -- needed for 32-bit I/O Port access


-- --------------------------------------------------------------------------
-- PCI Constants and Variables
-- --------------------------------------------------------------------------
constant SbPciConfig = #80009000        -- This is the base address 
                                        -- for all South Bridge
                                        -- PCI Configuration Index Registers.
    
constant PciIndexPort = #CF8            -- PCI Configuration Index Register
constant PciDataPort = #CFC             -- PCI Configuration Data Register
                                

-- --------------------------------------------------------------------------
-- This function reads and returns a 32-bit data item from the 
-- South Bridge - PCI Configuration Registers
-- --------------------------------------------------------------------------
function SB_PCI_READ(word FunctionNumber, word Index)
    atom Offset                         -- Euphoria type "atom" is required
                                        -- to hold 32-bit unsigned integers.

    Offset = SbPciConfig                -- Start with the Base Address.
           + (FunctionNumber * 256)     -- The arguments "FunctionNumber"
           + Index                      -- and "Index" which are passed into 
                                        -- this routine, are used to arrive 
                                        -- at the actual value used.

    outp32(PciIndexPort, Offset)        -- send calculated Offset to Port 
    return inp32(PciDataPort)           -- return the desired Data
end function


-- --------------------------------------------------------------------------
-- This procedure writes a 32-bit data item to the 
-- South Bridge - PCI Configuration Registers
-- --------------------------------------------------------------------------
procedure SB_PCI_WRITE(word FunctionNumber, word Index, atom Data)
    atom Offset                         -- Euphoria type "atom" is required
                                        -- to hold 32-bit unsigned integers.

    Offset = SbPciConfig                -- Start with the Base Address.
           + (FunctionNumber * 256)     -- The arguments "FunctionNumber"
           + Index                      -- and "Index" which are passed into 
                                        -- this routine, are used to arrive 
                                        -- at the actual value used.

    outp32(PciIndexPort, Offset)        -- send calculated Offset to Port 
    outp32(PciDataPort, Data)           -- send the desired Data
end procedure




-- --------------------------------------------------------------------------
-- Enable PCI Function #0 
--
-- There are four PCI "Functions" available in the ZFx86 chip: F0, F1, F2, F3
-- Functions F1, F2, & F3 must remain active at all times for proper
-- operation of the chip.
-- F0 can be disabled however, and in fact defaults to disabled after Reset.
-- 
-- This routine insures that it is Enabled, so that the GPIO features that
-- it controls can be used.  Part of the process of enabling this Function
-- requires Resetting parts of the chip.
-- --------------------------------------------------------------------------
atom ResetCtrlReg

ResetCtrlReg = SB_PCI_READ(0, #44)                      -- get current value

ResetCtrlReg = and_bits(ResetCtrlReg, #FFFFFFFE)        -- clear "Execute
                                                        -- System Wide Reset"
                                                        -- bit.  We don't
                                                        -- want to Reset the
                                                        -- whole machine, 
                                                        -- just parts of the
                                                        -- ZFx86 chip.

ResetCtrlReg = or_bits(ResetCtrlReg, #02000000)         -- enable PCI-F0

SB_PCI_WRITE(0, #44, ResetCtrlReg)                      -- write new values







-- --------------------------------------------------------------------------
-- Configure PCI Function #0 Base Address Register #0 Location
--
-- The registers used to control and manipulate the GPIO features are located
-- in I/O ports which are addressed as offsets from the contents of this 
-- "Base Address Register".
-- --------------------------------------------------------------------------
atom F0Bar0                             -- PCI Function #0 Base Address Reg 0

SB_PCI_WRITE(0, #10, #4000)             -- initialize F0Bar0 to known location
F0Bar0 = SB_PCI_READ(0, #10)            -- get current F0 Base Address #0
F0Bar0 = and_bits(F0Bar0, #0000FFFC)    -- keep only lower 16 bits, and make
                                        -- sure the 2 least significant bits
                                        -- are cleared to zeros.








-- --------------------------------------------------------------------------
-- Variables used to Access GPIO Features
-- --------------------------------------------------------------------------
atom DataOut0                           -- GPIO Data Output 0 Register
atom DataIn0                            -- GPIO Data Input 0 Register
word IntEnable0                         -- GPIO Interrupt Enable 0 Register
word Status0                            -- GPIO Status 0 Register
word PinCfgSelReg                       -- GPIO Pin Configuration Select Reg
word PinCfgAccReg                       -- GPIO Pin Configuration Access Reg
atom CfgData                            -- current Pin Configuration Data


DataOut0     = F0Bar0 + #00              
DataIn0      = F0Bar0 + #04              
IntEnable0   = F0Bar0 + #08              
Status0      = F0Bar0 + #0C
PinCfgSelReg = F0Bar0 + #20              
PinCfgAccReg = F0Bar0 + #24              





-- --------------------------------------------------------------------------
-- This Procedure Writes to GPIO Pins Configured as Outputs.
-- If the Data passed in = 0, then the pin is driven Low.
-- If the data passed in = 1, then the pin is driven High.
-- Any other data value, will retain the previous state of that bit.
--
-- A Sequence of 8 values is passed in to this procedure
-- Each element represents a GPIO Pin as follows: {0,1,2,3,4,5,6,7}
--
-- While the GPIO pins are numbered 0-7, Euphoria counts its sequence
-- elements starting with 1.  The manipulations below use the values 1-8
-- rather than 0-7, just to keep it simple.  This seemed cleaner than
-- having lots of references to pos[bit+1] etc.
-- --------------------------------------------------------------------------
procedure SET_GPIO_STATE(sequence NewValue)
   atom       CurrentData                
   atom       NewBits
   sequence   pos

   pos = {#01,#02,#04,#08,#10,#20,#40,#80}      -- position of bit in Byte

   CurrentData = inp32(DataOut0)                -- get state of GPIO pins
   NewBits = CurrentData                        -- some bits may not change
   for bit = 1 to 8 do
      if NewValue[bit] = 1 then
         NewBits = or_bits(NewBits, pos[bit])   -- turn Bit on
      elsif NewValue[bit] = 0 then
         NewBits = and_bits(NewBits, not_bits(pos[bit]))    -- turn Bit off
      end if
   end for
   outp32(DataOut0, NewBits)                    -- update all GPIO pins
end procedure




-- --------------------------------------------------------------------------
-- Configure the GPIO-6 pin as an input
--
-- We will prevent it from generating PME (Power Management Events).
-- It will use the built-in signal "de-bouncer" for input detection.
-- We will look for rising edges on this signal, since the default state
-- is "high".  A switch will pull the line low, and we will watch for it to 
-- go high again when the switch is released.
-- --------------------------------------------------------------------------
atom PME                                -- used to hold PME settings

PME = inp32(IntEnable0)                 -- get current settings
PME = and_bits(PME, not_bits(#40))      -- keep all except GPIO-6 as is
outp32(IntEnable0, PME)                 -- disable PME generation for GPIO-6

outp32(PinCfgSelReg, 6)                 -- Select GPIO-6
CfgData = inp32(PinCfgAccReg)           -- get current configuration
CfgData = and_bits(CfgData, #FFFFFF80)  -- clear PME & Output related bits
CfgData = or_bits(CfgData, #00000064)   -- Debounce, rising edge, pull-up
outp32(PinCfgAccReg, CfgData)           -- enable pin as input



-- --------------------------------------------------------------------------
-- Configure the GPIO-7 pin as an output
-- --------------------------------------------------------------------------
constant x = -1                         -- used to indicate "Don't Care"

SET_GPIO_STATE({x,x,x,x,x,x,x,0})       -- set the initial state for the
                                        -- GPIO-7 pin to 0 or OFF.  
                                        -- Reset causes all pins to default
                                        -- to a high state.

outp32(PinCfgSelReg, 7)                 -- Select GPIO-7
CfgData = inp32(PinCfgAccReg)           -- get current configuration
CfgData = and_bits(CfgData, #FFFFFF80)  -- clear PME & Output related bits
CfgData = or_bits(CfgData, 3)           -- configure as Push/Pull Output
outp32(PinCfgAccReg, CfgData)           -- enable pin as output



-- --------------------------------------------------------------------------
-- Start the actual application
--
-- Look for a rising edge on GPIO-6 Input, and
-- Send pulses (Blink the Light) to GPIO-7
-- until another rising edge is seen on GPIO-6,
-- at which time we turn the pulses off.
-- Pressing any key on the keyboard ends the program.
--
-- In addition to manipulating GPIO-7, the program also shows activity on
-- the screen, and "flashes" the P.O.S.T. code display (if present)
-- for diagnostic purposes.  This is just a back-up measure to show that
-- the switch activity was detected on GPIO-6.
--
-- Since we are only polling occasionally for changes to the Status Register,
-- it may not be recognized immediately.  Specifically, if the program is
-- sending outputs to GPIO-7, it will not see that the switch on GPIO-6 or
-- the keyboard has been pressed until it finishes the current burst of
-- three output pulses, and the LongDelay that follows them.  The Status
-- Register does latch the switch activity when it occurs in real time, but
-- this program won't react to it until it polls during the next loop.
-- --------------------------------------------------------------------------
constant ShortDelay = 1000000
constant LongDelay  = 5000000

integer Activated                       -- 0 = false, 1 = true

Activated = 0                           -- start out inactive
outp32(Status0, #40)                    -- Write a 1 to GPIO-6 bit to clear it

puts(1,"\n\nPress the button attached to GPIO-6 to activate the alarm")
puts(1,"\nPress it again to to turn the alarm OFF\n\n")

while get_key() = -1 do                 -- no key pressed yet
   
   if and_bits(inp32(Status0),#40) != 0 then   -- the switch has been pressed
      outp32(Status0, #40)              -- Write a 1 to GPIO-6 bit to clear it
      if Activated = 0 then
         Activated = 1                  -- turn the alarm ON
         puts(1,"\nAlarm Activated\n")
      else 
         Activated = 0                  -- turn the alarm OFF
         puts(1,"\n\nAlarm is now OFF, press any key to exit\n")
      end if
   end if

   if Activated then
      puts(1,"Light = ")                    -- simulate a blinking light  
      for Burst = 1 to 3 do
         puts(1,"1")                        -- the light goes ON
         SET_GPIO_STATE({x,x,x,x,x,x,x,1})  -- start a high pulse
         Output(#11, #80)                   -- write POST Code (if present)
         for Count = 1 to ShortDelay do     -- waste time
         end for
 
         puts(1,"0")                        -- the light goes out
         SET_GPIO_STATE({x,x,x,x,x,x,x,0})  -- start a low pulse
         Output(#00, #80)                   -- write POST Code (if present)
         for Count = 1 to ShortDelay do     -- waste time
         end for
      end for
      
      for Count = 1 to LongDelay do         -- pause between bursts
      end for
      puts(1,"\n")                          -- send "new line" to screen
   end if
end while
